{ *************************************************************************** }
{                                                                             }
{ Hotzonu Round Class Unit                                                    }
{                                                                             }
{ 2001-04 Hotzonu Project, Monazilla Software Foundation 2001                 }
{                                                                             }
{ *************************************************************************** }

unit Hzbround;

interface

uses
  Classes, SysUtils, ZonuClass, ZonuHttp, Main, Hzbclass;

type
  TZonuCheck  = class;
  TZonuUpdateCheck  = class;
  TZonuRound  = class;

  TZonuCheck =  class(TThread)
  private
  protected
    FListDoc: TZonuListDoc;
    FThreadItemList: TThreadItemList;
    FTarget: TList;
    FShowMsg: boolean;
    procedure ScreenUpdate; virtual; abstract;
    procedure OutputMessage(const Msg: string);
  public
    constructor Create(CreateSuspended: Boolean);
    destructor Destroy; override;
    procedure AddThread(ThreadItem: TThreadItem);
    procedure Abort; virtual; abstract;
    property  ThreadItemList: TThreadItemList read  FThreadItemList write FThreadItemList;
    property  ListDoc: TZonuListDoc read  FListDoc write FListDoc;
  end;

  TZonuUpdateCheck = class(TZonuCheck)
  private
    FTargetHost: string;
    procedure ThreadListDownload(Sender: TThreadListDownloader;
      ThreadItem: TThreadItem; Size, TotalSize: integer;
      var MoreData: boolean);
    procedure ThreadListError(Sender: TDownloader; ErrorCode: integer;
      ErrorMsg: string);
  protected
    procedure ScreenUpdate; override;
    procedure Execute; override;
  public
    procedure Abort; override;
  end;

  TZonuRound = class(TZonuCheck)
  private
    procedure ResEndDownload(Sender: TDownloader; Error: boolean);
    procedure ResDownload(Sender: TResDownloader; Res: TRes; Size,
      TotalSize: integer);
    procedure ResDownloadError(Sender: TDownloader; ErrorCode: integer;
      ErrorMsg: string);
  protected
    procedure ScreenUpdate ; override;
    procedure Execute; override;
  public
    procedure Abort; override;
  end;

implementation

uses
  Zonutils, MonaUtils, ZonuConf, Hzbconst, DolibModule;

////////////////////////////////////////////////////////////////////////////////
{ TZonuCheck }
////////////////////////////////////////////////////////////////////////////////

constructor TZonuCheck.Create(CreateSuspended: Boolean);
begin
  inherited;
  FThreadItemList :=  TThreadItemList.Create;
  FTarget         :=  TList.Create;
end;

destructor TZonuCheck.Destroy;
begin
  FThreadItemList.Clear;
  FThreadItemList.Free;
  FTarget.Free;
  inherited;
end;

procedure TZonuCheck.AddThread(ThreadItem: TThreadItem);
begin
  ThreadItem.AddRef;
  FThreadItemList.Add(ThreadItem)
end;

procedure TZonuCheck.OutputMessage(const Msg: string);
begin
  {  ** CALL IN SYNCHRONIZE **  }
  HotzonuWindow.Memo1.Lines.Add(Msg);
  {  ** CALL IN SYNCHRONIZE **  }
end;

////////////////////////////////////////////////////////////////////////////////
{ TZonuUpdateCheck }
////////////////////////////////////////////////////////////////////////////////

procedure TZonuUpdateCheck.ScreenUpdate;
var
  ActiveListDoc: TZonuListDoc;
begin
  {  ** CALL IN SYNCHRONIZE **  }
  try
    if  (ListDoc <> nil)  then  begin
      if  (HotzonuWindow.GetActiveListDoc(ActiveListDoc)) then  begin
        HotzonuWindow.ThreadListView.Invalidate;
      end;
    end;
    if  (FShowMsg) then  OutputMessage(Format('  `FbNF%s', [FTargetHost]));
  except
    //
  end;
  {  ** CALL IN SYNCHRONIZE **  }
end;

procedure TZonuUpdateCheck.Execute;
var
  Board: TBoard;
  Downloader: TThreadListDownloader;
  sl: TStringList;
  i, x, y : integer;
  List: TThreadItemList;
begin
  inherited;
  OutputMessage('XV`FbNJn');

  sl  :=  TStringList.Create;
  for i :=  0 to  FThreadItemList.Count - 1 do  begin
    Board :=  FThreadItemList[i].Board;
    if  (sl.IndexOf(Board.Url) < 0) then  begin
      sl.Objects[sl.Add(Board.Url)] := Board;
    end;
  end;

  for i :=  0 to  sl.Count - 1  do  begin
    FTargetHost :=  sl[i];
    FTarget.Clear;
    for x :=  0 to  FThreadItemList.Count - 1 do  begin
      Board :=  FThreadItemList[x].Board;
      if  (sl[i] = Board.Url)  then  begin
        if  (FThreadItemList[x].CanDownload)  then  begin
          FTarget.Add(FThreadItemList[x]);
          FThreadItemList[x].Downloading  :=  True;
        end;
      end;
    end;
    //
    //
    //
    if  (FTarget.Count > 0) then  begin
      FShowMsg  :=  True;
      Synchronize(ScreenUpdate);
      List      :=  TThreadItemList.Create;
      Board     :=  TBoard(sl.Objects[i]);
      Downloader                :=  TThreadListDownloader.Create;
      Downloader.Site           :=  Board.Site;
      Downloader.BBS            :=  Board.BBS;
      Downloader.Board          :=  Board;
      Downloader.ThreadItemList :=  List;
      Downloader.ViewObject     :=  nil;
      Downloader.CharSet        :=  Board.CharSet;

      Downloader.OnData         :=  ThreadListDownload;
      Downloader.OnError        :=  ThreadListError;

      try
        Board.BBS.DownloadThreadList(Downloader, True, False, False);
        //
        for x :=  0 to  FThreadItemList.Count - 1  do  begin
          for y :=  0 to  List.Count - 1  do  begin
            if  (FThreadItemList[x].BoardID = TThreadItem(List[y]).BoardID) and
                (FThreadItemList[x].DatNo  = TThreadItem(List[y]).DatNo) then
            begin
              if  (FThreadItemList[x].AgeCount <  TThreadItem(List[y]).AgeCount)  then  begin
                FThreadItemList[x].AgeCount :=  TThreadItem(List[y]).AgeCount;
                FThreadItemList[x].Updated  :=  True;
              end;
            end;
          end;
        end;
      except
        for x :=  0 to  FThreadItemList.Count - 1  do  begin
          if  (FThreadItemList[x].BoardID = Board.BoardID) then begin
            FThreadItemList[x].IsBroken :=  True;
          end;
        end;
      end;
      //
      Downloader.Free;
      List.Clear;
      List.Free;
    end;
    //
    for x :=  0 to  FTarget.Count - 1 do  begin
      TThreadItem(FTarget[x]).Downloading :=  False;
    end;

    if  (Terminated)  then  Break;
  end;
  FTarget.Clear;
  FTargetHost :=  '';
  FShowMsg    :=  False;
  Synchronize(ScreenUpdate);

  sl.Free;
  OutputMessage('XV`FbNI');
  OutputMessage(' ');
end;

procedure TZonuUpdateCheck.ThreadListDownload(
  Sender: TThreadListDownloader; ThreadItem: TThreadItem; Size,
  TotalSize: integer; var MoreData: boolean);
begin
  if  (ThreadItem <> nil) then  begin
    //ThreadItem.AddRef;
    Sender.ThreadItemList.Add(ThreadItem);
  end;
end;

procedure TZonuUpdateCheck.ThreadListError(Sender: TDownloader;
  ErrorCode: integer; ErrorMsg: string);
var
  Board : TBoard;
begin
  if  (ErrorCode <> 0)  then  begin
    Board :=  TThreadListDownloader(Sender).Board;
    if  (Board <> nil)  then  begin
      OutputMessage('Xbhꗗ擾G[' +
                    ' ' +  Board.Name + '' + Board.Site.SiteName);
      OutputMessage('' +  IntToStr(ErrorCode) + ':' + ErrorMsg);
      OutputMessage('');
    end;
  end;
end;

procedure TZonuUpdateCheck.Abort;
begin
  OutputMessage('XV`FbN𒆎~܂B΂炭҂B');
  Terminate;
end;

////////////////////////////////////////////////////////////////////////////////
{ TZonuRound }
////////////////////////////////////////////////////////////////////////////////

procedure TZonuRound.Execute;
var
  Downloader  : TResDownloader;
  ResList     : TResList;
  i: integer;
begin
  inherited;
  OutputMessage('XV`FbNJn');
  for i :=  0 to  FThreadItemList.Count - 1 do  begin
    if  (SameText(FThreadItemList[i].BBS.BBSID, BBS_DOLIB)) and
        (not DolibManager.Connected)  then
    begin
      if  (ZonuConfig.DolibUserName <> '')  then  begin
        DolibManager.Connect;
      end;
      Break;
    end;
  end;

  for i :=  0 to  FThreadItemList.Count - 1 do  begin
    FTarget.Clear;
    if  (
      (SameText(FThreadItemList[i].BBS.BBSID, BBS_DOLIB)) and
      (FThreadItemList[i].CanDownload) and
      (DolibManager.Connected)
        ) or (
      (not SameText(FThreadItemList[i].BBS.BBSID, BBS_DOLIB)) and
      (FThreadItemList[i].CanDownload)
        ) then
    begin
      FTarget.Add(FThreadItemList[i]);
      FThreadItemList[i].Downloading  :=  True;
      FShowMsg  :=  True;
      Synchronize(ScreenUpdate);
      //
      ResList                 :=  TResList.Create(FThreadItemList[i]);
      //
      Downloader              :=  TResDownloader.Create;
      Downloader.BBS          :=  FThreadItemList[i].BBS;
      Downloader.Site         :=  FThreadItemList[i].Site;
      Downloader.Board        :=  FThreadItemList[i].Board;
      Downloader.ThreadItem   :=  FThreadItemList[i];
      FThreadItemList[i].AddRef;
      Downloader.ResList      :=  ResList;
      Downloader.OriginalUrl  :=  '';
      Downloader.ViewObject   :=  nil;
      Downloader.NoCache      :=  False;
      Downloader.AutoPilot    :=  True;
      Downloader.CharSet      :=  FThreadItemList[i].Board.CharSet;
      Downloader.CacheReady   :=  False;
      Downloader.OnData       :=  ResDownload;
      Downloader.OnEnd        :=  ResEndDownload;
      Downloader.OnError      :=  ResDownloadError;
      FThreadItemList[i].LoadFromFile(ResList);
      //
      FThreadItemList[i].LoadCount := FThreadItemList[i].LoadCount + 1;
      //
      Downloader.CacheReady   :=  True;
      //
      try
        FThreadItemList[i].IsBroken :=  False;
        Downloader.BBS.DownloadThreadItem(Downloader,
                                          Downloader.NoCache,
                                          False,
                                          True,
                                          True
                                          );
      except
        FThreadItemList[i].IsBroken :=  True;
      end;
      //
      FThreadItemList[i].Downloading  :=  False;
      FShowMsg  :=  False;
      Synchronize(ScreenUpdate);
      //
      Downloader.Free;
      ResList.Free;
    end;
    if  (Terminated)  then  Break;
  end;
  OutputMessage('XV`FbNI');
  OutputMessage(' ');
end;

procedure TZonuRound.ResDownload(Sender: TResDownloader; Res: TRes;
  Size, TotalSize: integer);
begin
  Sender.ResList.Add(Res);
  if  (Sender.ThreadItem.NewCount < 0)  then  Sender.ThreadItem.NewCount := 0;
  Sender.ThreadItem.NewCount :=  Sender.ThreadItem.NewCount + 1;
end;

procedure TZonuRound.ResEndDownload(Sender: TDownloader;
  Error: boolean);
var
  dl  : TResDownloader;
begin
  dl  :=  TResDownloader(Sender);
  dl.ThreadItem.Downloading :=  False;
  if  (not Error) then  begin
    if  (dl.ResList.Count > 0)  then  begin
      if  (ZonuEnviron.Cached)  and (not Sender.NotModified) and (dl.ThreadItem.NewCount > 0) then  begin
        dl.ThreadItem.DownloadCount :=  dl.ThreadItem.DownloadCount + dl.ThreadItem.NewCount;
        dl.ResList.SaveToFile(dl.ThreadItem.LocalDatFileName);
        dl.ThreadItem.SaveConfig;
      end;
    end;
  end;
  ZonuConfig.LoadThreadCount    :=    ZonuConfig.LoadThreadCount + 1;
  TResDownloader(Sender).ThreadItem.ReleaseRef;
end;

procedure TZonuRound.ScreenUpdate;
var
  ActiveListDoc: TZonuListDoc;
  i: integer;
begin
  {  ** CALL IN SYNCHRONIZE **  }
  try
    if  (ListDoc <> nil)  then  begin
      if  (HotzonuWindow.GetActiveListDoc(ActiveListDoc)) then  begin
        if  (FTarget.Count > 0) then  begin
          for i :=  0 to ListDoc.ThreadItemList.Count - 1  do  begin
            if  (FTarget[0] = ListDoc.ThreadItemList[i])  then  begin
              HotzonuWindow.ThreadListView.Items[i].Update;
            end;
          end;
          if  (FShowMsg) then  OutputMessage(Format('  񒆁F%s', [TThreadItem(FTarget[0]).Title]));
        end;
      end;
    end;
  except
    //
  end;
  {  ** CALL IN SYNCHRONIZE **  }
end;

procedure TZonuRound.Abort;
begin
  OutputMessage('𒆎~܂B΂炭҂B');
  Terminate;
end;

procedure TZonuRound.ResDownloadError(Sender: TDownloader;
  ErrorCode: integer; ErrorMsg: string);
begin
  //
end;

end.
